import os.path, glob, drugrefapi_plugin
from pyPgSQL import PgSQL as dbapi

def queryresult2rowdict(queryresult):
	rows = []
	for row in queryresult:
		d = {}
		for f in row.keys():
			d[f]=row[f]
		rows.append(d)
	return rows



class drugref:
	def __init__(self, plugindir="plugins", name = "Drugref Service", version='1.0'):
		self._plugindir = plugindir
		self._plugins=[]
		self._services = {}
		self._provided = {}
		self._name = name
		self._version = version

                #DrugRef Gen 1
		self._db = "drugref2"
		self._user = None
		self._pwd = None

		#scan the plugin directory for plugins
		for file in glob.glob(os.path.join(plugindir, "*.py")):
			print "Importing module:", file
			try:
				module = drugrefapi_plugin.import_path(file)
				self._plugins.append(module)
                                print "module added ", module
			except:
				self._log("Could not import %s" % file)
		for plugin in self._plugins:
			#try:
			module = plugin.drugrefplugin()
			#except:
			#	self._log("Could not instantiate plugin module %s" % file)
			#	continue
			name, version, provides, theplugin = module.register()
			self._services[name] = {'version':version, 'plugin': theplugin, 'provides':provides}
			#for each keyword (attribute) that plugin provides
			for provided in provides:
                                print name, version, provides
				try:
					#if there is already some service providing this attribute,
					#add the new service as potential provider to the list
					self._provided[provided].append(name)
				except:
					#else just create a new list of service providers for that new attribute
					self._provided[provided] = [name]

	def identify(self):
		return self._name

	def version(self):
		return self._version

	def list_available_services(self):
		return self._services.keys()

	def list_capabilities(self, service):
		return self._services[service]['provides']

	def fetch(self, attribute, key, services=[], feelinglucky=1):
		try:
			providers = self._provided[attribute]
		except:
			return {'Error': '"%s" not provided by any registered service' % attribute}
		results = {}
		#do we just want to check certain services in order of preference?
		if len(services)>0:
			myservices = services
		#or shall we check all available services?
		else:
			myservices = self._services.keys()
		#checxk all services we want to check for the requested attribute for the stated key
		for service in myservices:
			module = self._services[service]
			result = module['plugin'].get(attribute, key)
			if len(result)>0:
				results[service] = result
				#if we want only the first hit from the preferred provider, nothing else to do:
				if feelinglucky:
					return results
		return results


	def _log(self, msg):
		print msg


        #NEW FUNCTION
        def get(self, attribute, key):
                api = holbrook(database='drugref')
	        #interactions =  api.get('interactions_byATC', ['N02BE01','N05BA01', 'N05BA12'])
                interactions =  api.get('interactions_byATC', key)
                #if interactions:
	        #        for interaction in interactions:
		#                print interaction['affectingatc'], " (", interaction['affectingdrug'], ") affects", interaction['affectedatc'], " (", interaction['affecteddrug'] , ")"
                #                print interaction['affectingatc'], " ", interaction['effect'], " ", interaction['affectedatc']
                #else:
                #        print "No Interactions"
                return interactions


	def list_search_element(self, searchStr):
                """
                input String: String to search the database for (Ingredient, Generic, Brand)
		returns all matching search element names , ids and categories for the given searchString.
                {'id':'0','category':'','name':'None found'}
                Search is case insensitive
                """
                con = dbapi.connect(database=self._db)
                cur = con.cursor()

		searchStr = searchStr.replace(","," ");
		searchStr = searchStr.replace("'","");

		seq = searchStr.split()
		searchStr = "%' and name ilike '%".join(seq)
                query = "select id,category,name from cd_drug_search where name ilike '%%%s%%' order by name" % searchStr
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]




	def list_search_element_route(self, searchStr, route):
                """
                input searchStr: String to search the database for (Ingredient, Generic, Brand)
		input route: String to limit search by drug administration route
		returns all matching search element names , ids and categories for the given searchString.
                {'id':'0','category':'','name':'None found'}
                Search is case insensitive
                """
                con = dbapi.connect(database=self._db)
                cur = con.cursor()

		searchStr = searchStr.replace(","," ");
		searchStr = searchStr.replace("'","");
		route = route.replace(","," ");
		route = route.replace("'","");

		seq1 = searchStr.split()
		seq2 = route.split()
		searchStr = "%' and name ilike '%".join(seq1)
		route = "%' or route_of_administration ilike '%".join(seq2)
                query = "select id,category,name from cd_drug_search where name ilike '%%%s%%' and drug_code in (select drug_code from cd_route where route_of_administration ilike '%%%s%%') order by name" % (searchStr, route)
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]



	def list_brands_from_element(self,drug_id):
                """
                input String: id of generic drug or drug class
                returns all names. ids and categories for the inputed id
                """
		con = dbapi.connect(database=self._db)
                cur = con.cursor()

                drug_code = ""
                cat = ""
                code_cat_query = "select drug_code, category from cd_drug_search where id = '%s' " % drug_id
                print code_cat_query
                cur.execute(code_cat_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                drug_code = code['drug_code']
                                cat = code['category']
                else:
                        return [{'drug_code':'0','category':''}]

                query = ""
                print  " drug_code %s cat %s " % ( drug_code, cat )


                if cat == 8:
                   query = "select sd.id, sd.category , sd.name   from cd_therapeutic_class tc, cd_drug_search sd where tc.tc_atc_number = '%s' and tc.drug_code = sd.drug_code order by name" % drug_code
                elif cat == 10:
                   query = "select sd.id, sd.category , sd.name   from cd_therapeutic_class tc, cd_drug_search sd where tc.tc_ahfs_number = '%s' and tc.drug_code = sd.drug_code order by name" % drug_code
                else: # cat == 11 or 12
                   query = "select sd.id, sd.category , sd.name   from link_generic_brand lgb, cd_drug_search sd where lgb.id = '%s' and lgb.drug_code = sd.drug_code order by name" % drug_code
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]


	def list_search_element_select_categories(self, searchStr,categories):
                """
                input String: String to search
                input categories:  categories to search for
                returns all matching search element names , ids and categories for the given searchString.
                {'id':'0','category':'','name':'None found'}
                Search is case insensitive
                """
                con = dbapi.connect(database=self._db)
                cur = con.cursor()

		searchStr = searchStr.replace(","," ")
		searchStr = searchStr.replace("'","")
		seq = searchStr.split()
		searchStr = "%' and name ilike '%".join(seq)

                s = " and ( "
		catCount = 0
                for cate in categories:
			catCount = catCount +1
			if (catCount > 1 ):
                        	s =  "%s  or  category = '%s' " %  (s,cate)  #used to be d.category
			else:
				s = "%s category = '%s' " % (s,cate)  # used to be d.category
                if (catCount == 0):
        		s = ""
		else:
			s = "%s ) " % s

		query = "select id, category , name from cd_drug_search where  name ilike '%%%s%%' %s order by category,name" % (searchStr, s)
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]


        def get_allergy_warnings(self,atcCode,allergies):
                """
                input String: atccode of the drug to check for allergies
                input Dict: {'category':'','description':'','id':''}
                returns Dict: {'warnings':returnList}  of the ids that have reactions with drug.
                """
                con = dbapi.connect(database=self._db)
                cur = con.cursor()


                print "checking allergies for %s " % (atcCode)

                returnList = []

                if  atcCode == "" or atcCode == "null" :
                        return [{'warnings':returnList}]

                for ally in allergies:
                        alleHash = ally;
                        aType = alleHash['type']
                        aDesc = alleHash['description']
                        aId = alleHash['id']
                        print aId

                        """Drug Classes"""
                        if ( aType == '8'):
                                query = "select tc_atc_number from cd_therapeutic_class where tc_atc_number = '%s' and tc_atc = '%s' " % (atcCode,aDesc)
                                print query
                                cur.execute(query)
                                res = cur.fetchall()
                                if len(res)>0:
                                        print " %s is in this Allergy Group %s" % (atcCode,aDesc)
                                        returnList.append(aId)
                                else:
                                        print "%s Not in THis Group %s " % (atcCode,aDesc)
                        elif  ( aType == '10'):
                                query = "select tc_atc_number from cd_therapeutic_class where tc_atc_number = '%s' and tc_ahfs = '%s' " % (atcCode,aDesc)
                                print query
                                cur.execute(query)
                                res = cur.fetchall()
                                if len(res)>0:
                                        print " %s is in this Allergy Group %s" % (atcCode,aDesc)
                                        returnList.append(aId)
                                else:
                                        print "%s Not in THis Group %s " % (atcCode,aDesc)

                        elif  ( aType == '14' ):
                                print "Not yet implemented"
                                """
                                query = "select distinct  n.name, n.id_drug_element  from link_product_component pc, drug_element_name n  where  pc.id_component = n.id_drug_element and id_product in (select id from link_drug_atc where atccode  = '%s')" % atcCode
                                cur.execute(query)
                                res = cur.fetchall()
                                if len(res) >0:
                                        print "SEARCHING COMPONENTS OF %s for %s " % (atcCode,aDesc)
                                        for row in res:
                                                comp = row['name']
                                                if ( comp == aDesc ):
                                                        print "   FOUND %s " % row['name']
                                                        returnList.append(aId)
                                                else:
                                                        print "   NOT FOUND %s " % row['name']
                                else:
                                        print "NO COMPONENTS FOUND"
                                """
                        elif ( aType == '11' or aType == '12' ):
                                query = "select tc.tc_atc_number  from cd_drug_search ds, link_generic_brand lgb, cd_therapeutic_class tc where tc.tc_atc_number = '%s' and ds.name = '%s' and ds.drug_code = lgb.id and lgb.drug_code = tc.drug_code" % (atcCode,aDesc)
                                print query
                                #cur.execute(query,atcCode,aDesc)
                                cur.execute(query)
                                print "was there an exception thrown"
                                res = cur.fetchall()
                                if len(res)>0:
                                        print "WARNING ALLERGIC to %s " % aDesc
                                        returnList.append(aId)
                                else:
                                        print "NO WARNING FOR %s " % aDesc

                        elif ( aType == '13' ):
                                query = "select tc_atc_number from cd_drug_search ds, cd_therapeutic_class tc where name = '%s' and ds.drug_code = tc.drug_code and tc_atc_number = '%s'"
                                print query
                                cur.execute(query,atcCode,aDesc)
                                res = cur.fetchall()
                                if len(res)>0:
                                        print "WARNING ALLERGIC to %s " % aDesc
                                        returnList.append(aId)
                                else:
                                        print "NO WARNING FOR %s " % aDesc


                        else:
                                print "NOT MATCHED YET desc %s type %s atccode %s" % (aDesc,aType,atcCode)


                return [{'warnings':returnList}]
###################################################3


	def get_drug(self, pKey, html):
		"""
		highest level API function to retrieve a complete drug monograph (except for packaging, pricing and subsidy information)
        	struct with at least the following keys:
		name: string. International nonproprietary name (INPN) of this drug (="generic")
		atc:string. ATC Code
                essential: True if this drug is on the WHO essential drug list
		product: string. If this drug is not a generic, the product brand name is listed under this key, else this key is not available
		action: string. Description of mode of action.
		indications: array of structs. Each struct has "indication" as key, and a struct as value containing the following keys:
			- code : integer. Drugref condition code primary key
			- firstline : boolean. True if for this indication this drug is considered a first line treatment.
			- comment : string
		contraindications: array of structs. Each struct has "contraindication" as key, and a struct as value containing the following keys:
			- code : integer. Drugref condition code primary key.
			- severity : integer (1-3, 3 being absolute contraindication)
			- comment : string
		practice_points: array of strings
		paediatric_use:  string. Describing special considerations in paediatric use
		pregnancy_use: struct with following keys:
			- code : character. ADEC category
			- comment : string
		lactation_use: struct with following keys:
			* code : integer. 1=compatible, 2=restricted, 3=dangerous
			* comment : string
		renal_impairment: struct with following keys:
			- code : integer. 1=compatible, 2=restricted, 3=dangerous
			- comment : string
		hepatic_impairment: struct with following keys:
			- code : integer. 1=compatible, 2=restricted, 3=dangerous
			- comment : string
		common_adverse_effects: array of strings
		rare_adverse_effects: array of strings
		dosage: array of structs with following keys:
			text: string. If this key is available, only free text dosage information is available, as described in this string.
			* indication : integer. Drugref condition code primary key. 0 is wild card, indicating general dosage recommendation.
    			* units: string. SI unit for this dosage
    			* calculation_base_units: integer. 0=not applicable, 1=age in months, 2=age in years, 3=kg body weight, 4=cm2 body surface
    			* calculation_base : real. Meaning depends on calculation_base_units
    			* starting_range : real. Usual mimimal recommended dosage
    			* upper_range : real. Usual maximal recommended dosage
    			* frequency_units : integer. 0=not applicable, 1=seconds, 2=minutes, 3=hours, 4=days, 5=weeks, 6=months, 7=years
    			* frequency : integer. How often this drug should be administered
    			* duration_units: integer. Same as frequency_units, additional value 8='times'
    			* duration_minimum ; integer. How long a usual course of this drug should be given. -1 is "permanent", -2="p.r.n."
    			* duration_maximum : integer. -1 is "permanent", "-2"="p.r.n."
    			* constrained: boolean. If true, no automazied dosage suggestion must be generated, prescriber must read comment. (e.g. dosage per body surface etc.)
    			* comment
		"""
		con = dbapi.connect(database=self._db)
		cur = con.cursor()
		productId = ""
                origId = pKey

                drug_code_query = "select drug_code from cd_drug_search where id = '%s' " % (pKey)

                print drug_code_query
                cur.execute(drug_code_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                pKey = code[0]

                print pKey

                name_query = "select ingredient as name from cd_active_ingredients where drug_code = '%s' " % (pKey)
                print name_query

                atc_query  = "select tc_atc_number as atccode from cd_therapeutic_class where drug_code = '%s'" % (pKey)
                print atc_query
        	essential_query = ""
                #product_query = "select brand_name as name from cd_drug_product where drug_code = '%s'" % (pKey)
                product_query = "select  name from cd_drug_search where id =  '%s'" % (origId)
                print product_query
        	action_query = ""
		indications_query = ""
        	contraindications_query = ""
        	practice_points_query = ""
        	paediatric_use_query = ""
        	pregnancy_use_query = ""
        	lactation_use_query = ""
        	renal_impairment_query = ""
        	hepatic_impairment_query = ""
        	common_adverse_effects_query = ""
        	rare_adverse_effects_query = ""
        	dosage_query = ""

                regional_identifier_query = "select drug_code as id, drug_identification_number as comment from cd_drug_product where drug_code = '%s'" % (pKey)
                print regional_identifier_query


		name = ""
                atc  = ""
                essential = ""
                product = ""
                action = ""
                indications = ""
                contraindications = ""
                practice_points = ""
                paediatric_use = ""
                pregnancy_use = ""
                lactation_use = ""
                renal_impairment = ""
                hepatic_impairment = ""
                common_adverse_effects = ""
                rare_adverse_effects = ""
                dosage = ""
                regional_identifier = ""
		components = []

		cur.execute(name_query)
                res = cur.fetchall()
		if len(res)>0:
                        for code in res:
             			name = code[0]
		cur.execute(atc_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                atc = code[0]
		cur.execute(product_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                product = code[0]

		cur.execute(regional_identifier_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
				productId = code[0]
                                regional_identifier = code[1]


                component_query = "select ingredient as name, strength, strength_unit as unit from cd_active_ingredients where drug_code ='%s' " % (pKey)
                print component_query

		cur.execute(component_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
				ha = {}
				ha['name'] = code[0]
                                ha['strength'] = float(code[1])
                                ha['unit'] = code[2]
				components.append(ha)


		returnRows = []
		returnVal = {}
		returnVal['name'] = name
		returnVal['atc']  = atc
		returnVal['product'] = product
		returnVal['regional_identifier'] = regional_identifier
		returnVal['components'] = components
		returnRows.append(returnVal)
                print len(returnRows)
		return returnRows

	def get_generic_name(self, drug_id):
                con = dbapi.connect(database=self._db)
                cur = con.cursor()

                query = "select id, category , name from cd_drug_search where drug_code in (select lgb.id from link_generic_brand lgb ,cd_drug_search ds where ds.drug_code = lgb.drug_code and ds.id = '%s');" % drug_id

                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id_class':'0','name':'None found'}]

	def get_form(self, pKey):
		con = dbapi.connect(database=self._db)
		cur = con.cursor()

		query = "select pharm_cd_form_code, pharmaceutical_cd_form from cd_form where drug_code = (select drug_code from cd_drug_search where id='%s');" % pKey

		print query
		cur.execute(query)
		res = cur.fetchall()
		if len(res)>0:
			return queryresult2rowdict(res)
		else:
			return [{'pharm_cd_form_code':'','pharmaceutical_cd_form':'None found'}]


	def list_drug_class(self, Dclass):
                """
                input List: Dclass a list of drug_id to get drug classes for.
                new implementation: a list of id's is passed in that matches brand names need to find the id and name of the ahfs
                return Dict: {'id_drug':'0','id_class':'0','name':'None found'}
                """
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                whereString = " and ( "
                catCount = 0
                for item in Dclass:
                        catCount = catCount + 1
                        if (catCount > 1 ):
                                #whereString =  "%s  or  l.id_drug = '%s' " %  (whereString,item)
                                whereString =  "%s  or  sd1.id = '%s' " %  (whereString,item)
                        else:
                                whereString = "%s sd1.id = '%s' " % (whereString,item)
                if (catCount == 0):
                        whereString = ""
                else:
                        whereString = " %s ) " % whereString

                query = "select sd1.id as id_drug, sd2.id as id_class, sd2.name as name from cd_drug_search sd1, cd_therapeutic_class th, cd_drug_search sd2 where  sd1.drug_code = th.drug_code and th.tc_ahfs_number = sd2.drug_code %s order by sd2.id " % whereString


                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id_drug':'0','id_class':'0','name':'None found'}]



if __name__ == "__main__":
	import sys, time, DocXMLRPCServer as rpcserver
	#import sys, time, SimpleXMLRPCServer as rpcserver
	#parse the command line:
	drugrefserver = drugref()
	try:
		server_url = sys.argv[1]
	except:
		server_url = "localhost"
	try:
		server_port = int(sys.argv[2])
	except:
		server_port = 8001

	print "Serving at port", server_port, server_url
	server=rpcserver.DocXMLRPCServer((server_url, server_port))
        #server=rpcserver.SimpleXMLRPCServer((server_url, server_port))

 	try:
 		#should work from Python 2.3 onwards
 		server.register_introspection_functions()
 	except:
 		print "Server not capable of introspection"

 	#try:
 	#	server.register_multicall_functions()
 	#except:
 	#	print "Server not capable of multicall functionality"

	server.register_function(drugrefserver.fetch)
	server.register_function(drugrefserver.list_available_services)
	server.register_function(drugrefserver.list_capabilities)
	server.register_function(drugrefserver.identify)
	server.register_function(drugrefserver.version)

        ### Canadian Drugref gen 1 functions

        server.register_function(drugrefserver.get)
        server.register_function(drugrefserver.list_search_element)
        server.register_function(drugrefserver.list_search_element_route)
        server.register_function(drugrefserver.list_brands_from_element)
        server.register_function(drugrefserver.list_search_element_select_categories)
        server.register_function(drugrefserver.get_drug)
        server.register_function(drugrefserver.get_generic_name)
	server.register_function(drugrefserver.get_form)
        server.register_function(drugrefserver.list_drug_class)
        server.register_function(drugrefserver.get_allergy_warnings)

	try:
		print "Server started on", time.asctime()
		server.serve_forever()
	except:
		#exit gracefully, release port
		del(server)
		print "Server closed on", time.asctime()
